//====================================================
// Reflection Refraction
//====================================================
// By EVOLVED
// www.evolved-software.com
//====================================================

//--------------
// un-tweaks
//--------------
   matrix WorldVP:WorldViewProjection; 
   matrix World:World;   
   matrix ViewInv:ViewInverse; 
 
//--------------
// tweaks
//--------------
   float ReflectFactor=0.1f;
   float RefractAmount=1.0f;
   float3 RefractColor={1.2f,1.2f,1.0f}; 
   float3 ReflectColor={1.0f,1.0f,1.2f}; 
   float Alpha=1.0f;
   float4 FogColor={0.8f,0.8f,0.8f,1.0f};
   float FogRange=2500.0f;

//--------------
// Textures
//--------------
   texture ScreenTX <string Name="";>;	
   sampler2D Screen=sampler_state
      {
	Texture = <ScreenTX>;
   	ADDRESSU = CLAMP;
   	ADDRESSV = CLAMP;
   	ADDRESSW = CLAMP;
      };
   texture cubeMapTX : ENVIRONMENT <string Name = "";>;
   samplerCUBE cubeMap = sampler_state 
      {
	Texture = <cubeMapTX>;
      };

//--------------
// structs 
//--------------
   struct input
     {
 	float4 Pos:POSITION; 
	float3 Normal:NORMAL;
     };
   struct output
     {
 	float4 OPos:POSITION; 
        float3 ViewPos:TEXCOORD0;
	float3 normal:TEXCOORD1;
	float2 vNormal:TEXCOORD2;
        float4 Proj:TEXCOORD3;
	float Fog:FOG;
     };

//--------------
// vertex shader
//--------------
   output VS(input IN)
     {
 	output OUT;
	OUT.OPos=mul(IN.Pos,WorldVP); 
	float3 WPos=mul(IN.Pos,World);
 	float3 ViewPos=ViewInv[3].xyz-WPos;
	OUT.ViewPos=ViewPos;
	OUT.normal=normalize(mul(IN.Normal,World)); 
	OUT.vNormal=normalize(mul(IN.Normal,WorldVP)); 
        OUT.Proj=float4(OUT.OPos.x*0.5+0.5*OUT.OPos.w,0.5*OUT.OPos.w-OUT.OPos.y*0.5,OUT.OPos.w,OUT.OPos.w);
	OUT.Fog=1-saturate(dot(ViewPos/FogRange,ViewPos/FogRange));
        return OUT;
     }

//--------------
// pixel shader
//--------------
   float4 PS(output IN) : COLOR 
     {
        float2 projUV=IN.Proj.xy/IN.Proj.z;
        float angle=1-saturate(dot(IN.normal,normalize(IN.ViewPos))); 
        float2 nuv=projUV+IN.vNormal*(angle*RefractAmount);
        float3 refract=tex2D(Screen,nuv)*RefractColor;
    	float3 lookup=reflect(-IN.ViewPos,IN.normal);
  	float3 cube=texCUBE(cubeMap,lookup)*ReflectColor;
        return float4(lerp(refract,cube,saturate(angle+ReflectFactor)),Alpha);
     }

//--------------
// techniques   
//--------------
    technique ReflectionRefraction
      {
 	pass p1
      {		
 	VertexShader = compile vs_2_0 VS(); 
 	PixelShader  = compile ps_2_0 PS(); 
	FOGCOLOR=(FogColor); 
	FOGENABLE=TRUE;			
      }
      }


